@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.22  -- Rateup
+ MRTG 2.9.27  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -44,11 +44,19 @@
 #ifndef GFORM_GD
 #define GFORM_GD gdImagePng 
 #endif
-/* BSDI 2.0.1 does not have malloc.h */
-#if !defined(bsdi) && !defined(__FreeBSD__) && !defined(__OpenBSD__)
+
+/* BSD* does not have/need malloc.h */
+#if !defined(bsdi) && !defined(__FreeBSD__) && !defined(__OpenBSD__) && !defined(__APPLE__)
 #include <malloc.h>
 #endif
 
+/* MSVCRT.DLL does not know %ll in printf */ 
+#ifdef __MINGW32_VERSION
+#define LLD "%I64d"
+#else
+#define LLD "%lld"
+#endif
+
 /* WATCOM C/C++ 10.6 under Win95/NT */
 /* VC++ 6.0 under Win95/NT */
 #if defined(__WATCOMC__) || defined(WIN32)
@@ -61,7 +69,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.22";
+char *VERSION = "2.9.27";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -116,7 +124,7 @@ int col_inm[3];
 int col_outm[3];
 int col_outp[3];
 
-long	kilo = (long)1000;
+long   long kilo = (long long)1000;
 char*   kMG = (char*) NULL;
 char*   kMGcopy = (char*) NULL; /* JML bugfix */
 int	kMGnumber = 0;
@@ -126,7 +134,7 @@ int	kMGnumber = 0;
 
 struct HISTORY {
     time_t time;
-    unsigned long in, out, percent, inmax, outmax;
+    long long  in, out, percent, inmax, outmax;
 } *history;
 int Mh;
 
@@ -189,13 +197,13 @@ for debuging
 	inr  += (double) history[n].in * interval;\
 	outr += (double) history[n].out * interval;\
 	avc += interval;\
-	inmax =  max(inmax, history[n].inmax);\
-	outmax = max(outmax,history[n].outmax);\
+       inmax =  (long long) max(inmax, history[n].inmax);\
+       outmax = (long long) max(outmax,history[n].outmax);\
 	if (nextnow <= history[n+1].time) n++; else break;\
       } while (n < histvalid && nextnow < history[n].time);\
 \
       if (avc != steptime) {\
-	fprintf(stderr,"ERROR: StepTime does not match Avc %8lu. Please Report this.\n", avc);\
+       fprintf(stderr,"ERROR: StepTime does not match Avc %8llu. Please Report this.\n", avc);\
       }\
 \
       inr /= avc; outr /= avc;\
@@ -205,9 +213,9 @@ for debuging
  
 static void image(file,maxvi,maxvo,maxx,maxy,xscale,yscale,growright,step, bits, ytics, yticsf, peak)
 char *file;
-long maxvi,maxvo;
+long long  maxvi,maxvo;
 long maxx;
-unsigned long maxy,growright,step,bits;
+long maxy,growright,step,bits;
 double xscale,yscale;
 int ytics; /* number of tics on the y axis*/
 double yticsf; /* scale everything on the y axis with this factor*/
@@ -216,17 +224,17 @@ int peak;
     FILE *fo;
     int i,x,y,n,type;
     
-    long maxv;
-    unsigned long origmaxvi, origmaxvo,maxs, avc, inmax, outmax;
+    long long maxv;
+    long long origmaxvi, origmaxvo,maxs, avc, inmax, outmax;
     double inr, outr,muli, interval;
     time_t now,onow,nextnow;
     struct tm tm2, *tm=&tm2;
     char **graph_label;
     char ylab[30];
     /* scaling helpers */
-    long maxv_q;
-    unsigned long valsamp,maxin,maxout,digits,maxpercent=0;
-    unsigned long  sca_ten,sca_hun,nmax_q;
+    long long maxv_q;
+    long long valsamp,maxin,maxout,digits,maxpercent=0;
+    long long sca_ten,sca_hun,nmax_q;
     double avmxin,avmxout, avin, avout,latestout=0,latestin=0,nmax,avpercent=0,latestpercent=0;
     double nex_ten,nex_hun,nex_rnd;
     double sca_max_q,dummy;
@@ -237,12 +245,12 @@ int peak;
     struct HISTORY *lhist;
     /* ################################################# */
     /* Some general definitions for the graph generation */
-#define XSIZE (unsigned long)((maxx*xscale)+100+((options & OPTION_DORELPERCENT) ? 1 : 0)*30)
-#define YSIZE (unsigned long)((maxy*yscale)+35)
+#define XSIZE (long)((maxx*xscale)+100+((options & OPTION_DORELPERCENT) ? 1 : 0)*30)
+#define YSIZE (long)((maxy*yscale)+35)
     /* position the graph */
-#define ytr(y) (unsigned long)(maxy*yscale+14-((y)*yscale))
+#define ytr(y) (long)(maxy*yscale+14-((y)*yscale))
     /* translate x/y coord into graph coord */
-#define xtr(x) (unsigned long)((growright) ? (maxx*xscale+81-((x)*xscale)) : (81+((x)*xscale))) 
+#define xtr(x) (long)((growright) ? (maxx*xscale+81-((x)*xscale)) : (81+((x)*xscale))) 
     /* ################################################# */ 
 
     /* GD LIB declarations */
@@ -263,9 +271,9 @@ int peak;
     muli = 7*bits+1;
     maxvi *= muli;
     maxvo *= muli;
-    origmaxvi = labs(maxvi);
-    origmaxvo = labs(maxvo);
-    maxv = max(maxvi,maxvo);
+    origmaxvi = maxvi < (long long)0 ? -maxvi : maxvi;
+    origmaxvo = maxvo < (long long)0 ? maxvo : maxvo;
+    maxv = (long long) max(maxvi,maxvo);
     if (step > MONTH_SAMPLE) {
       type = 4;
       now = (long)(NOW / YEAR_SAMPLE)*YEAR_SAMPLE;
@@ -301,15 +309,15 @@ int peak;
 	avmxout += outmax;
 	
 	if (peak) {
-		maxin = max(maxin,inmax);
-    	        maxout = max(maxout,outmax);
-	        maxs = max(maxs,inmax);
-	        maxs = max(maxs,outmax);
-	} else {
-		maxin = max(maxin,inr);
-		maxout = max(maxout,outr);
-		maxs = max(maxs,inr);
-		maxs = max(maxs,outr);	
+               maxin = (long long) max(maxin,inmax);
+               maxout = (long long) max(maxout,outmax);
+               maxs = (long long) max(maxs,inmax);
+               maxs = (long long) max(maxs,outmax);
+       } else {
+               maxin = (long long) max(maxin,inr);
+               maxout = (long long) max(maxout,outr);
+               maxs = (long long) max(maxs,inr);
+               maxs = (long long) max(maxs,outr);      
 	}
 	if ((options & OPTION_DORELPERCENT) && outr) {
 		dummy = (double)100.*inr/outr;
@@ -332,23 +340,23 @@ int peak;
 	avmxout /= valsamp;
     }
 	
-    printf("%lu\n",(unsigned long)(maxin/muli+.5));
-    printf("%lu\n",(unsigned long)(maxout/muli+.5));
+    printf("" LLD "\n",(long long)(maxin/(long long)muli+.5));
+    printf("" LLD "\n",(long long)(maxout/(long long)muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(maxpercent+.5));
+       printf("" LLD "\n",(long long)(maxpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(avin/muli+.5));
-    printf("%lu\n",(unsigned long)(avout/muli+.5));
+    printf("" LLD "\n",(long long)(avin/(long long)muli+.5));
+    printf("" LLD "\n",(long long)(avout/(long long)muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(avpercent+.5));
+       printf("" LLD "\n",(long long)(avpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(latestin/muli+.5));
-    printf("%lu\n",(unsigned long)(latestout/muli+.5));
+    printf("" LLD "\n",(long long)(latestin/(long long)muli+.5));
+    printf("" LLD "\n",(long long)(latestout/(long long)muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(latestpercent+.5));
+       printf("" LLD "\n",(long long)(latestpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(avmxin/muli+.5));
-    printf("%lu\n",(unsigned long)(avmxout/muli+.5));
+    printf("" LLD "\n",(long long)(avmxin/(long long)muli+.5));
+    printf("" LLD "\n",(long long)(avmxout/(long long)muli+.5));
     
     if (maxv < 0 || (unsigned long) maxv < maxs) {maxv = maxs;}
       
@@ -531,19 +539,19 @@ int peak;
 	inr *=muli; outr *=muli; inmax *=muli; outmax *=muli;		
 
 	y = ((double)inr/nmax) * maxy;
-	if ((unsigned long) y >= maxy) y = maxy;
+	if (y >= maxy) y = maxy;
 	lhist[x].in = y;
 	y = ((double)outr/nmax) * maxy;
-	if ((unsigned long) y >= maxy) y = maxy;
+	if (y >= maxy) y = maxy;
 	lhist[x].out = y;
 	y = ((double)inmax/nmax) * maxy;
-	if ((unsigned long) y >= maxy) y = maxy;
+	if (y >= maxy) y = maxy;
 	if (x && (unsigned long) y < (lhist[x-1].inmax)) {
 	  y += (lhist[x-1].inmax); y /= 2;
 	}
         lhist[x].inmax = y;
 	y = ((double)outmax/nmax) * maxy;
-	if ((unsigned long) y >= maxy) y = maxy;
+	if (y >= maxy) y = maxy;
 	if (x && (unsigned long) y < (lhist[x-1].outmax)) {
 	  y += (lhist[x-1].outmax); y /= 2;
 	}
@@ -552,22 +560,16 @@ int peak;
 	if (options & OPTION_DORELPERCENT) {
 	   if (outr != (unsigned long)0) {
               percent = (double)inr/(double)outr;
-/*
-	   if (lhist[x].out != (unsigned long)0) {
-              percent = (double)lhist[x].in/(double)lhist[x].out;
-	   if (lhist[x].in != (double)0.) {
-              percent = (double)lhist[x].out/lhist[x].in;
-*/
 	   }
 	   else {
               percent = (double)0.;
 	   }
 	   if (percent > (double)1) percent = (double)1.;
 	   percent *= maxy;
-           lhist[x].percent = (unsigned long)percent;
+           lhist[x].percent = (long long)percent;
 	}
 	else {
-           lhist[x].percent = (unsigned long)0;
+           lhist[x].percent = (long long)0;
 	}
 	
 	now -= step;
@@ -664,7 +666,7 @@ int peak;
 	if (!bits) { longup = "Bytes per Second"; shortup = "Bytes/s"; }
 	else { longup = "Bits per Second"; shortup = "Bits/s"; }
     }
-    if (maxy < (unsigned long) gdFontSmall->w*16) {	
+    if (maxy < gdFontSmall->w*16) {	
 	gdImageStringUp(graph, gdFontSmall,8, ytr((maxy-gdFontSmall->w*strlen(shortup))/2),
 		      (unsigned char*)shortup, i_grid);
     } else {
@@ -825,10 +827,10 @@ static int readhist(file)
 char *file;
 {
     FILE *fi;
-    int i,x, retcode = 0;
+    int x, retcode = 0;
     char buf[256];
     struct HISTORY *hist;
-    unsigned long rd[5];
+    long long rd[5];
     time_t cur;
 
     if ((fi = fopen(file,"r")) != NULL) {
@@ -841,13 +843,16 @@ char *file;
 	hist = history;
 	while(!feof(fi)) {
 	    fgets(buf,256,fi);
-	    if (sscanf(buf,"%lu %lu %lu %lu %lu",
+           if (sscanf(buf,"" LLD " " LLD " " LLD " " LLD " " LLD "",
                  &rd[0],&rd[1],&rd[2],&rd[3],&rd[4]) <5 ) {
 	      rd[3]=rd[1];rd[4]=rd[2]; }
+
+/* we are long long now, so don't cut bit 8
 	    for (i=0;i<=4;i++){
 	      if (rd[i] & 0x80000000)
 		rd[i] = 0;
 	    }
+*/
 					
 	    hist->time=rd[0];
 	    hist->in=rd[1];
@@ -942,14 +947,14 @@ static void readfile() {
 	     
 static void update(in, out, abs_max, absupdate)
 char *in, *out;
-unsigned long abs_max;
+long long abs_max;
 int absupdate;
 {
     FILE *fo;
     char buf[128],buf1[128],buf2[128];
     time_t now,nextnow,plannow;
-    long inrate, outrate;
-    unsigned long avc, inmax, outmax;
+    long long inrate, outrate;
+    long long avc, inmax, outmax;
     double period, interval;
     int x,n,nout;
     struct HISTORY *lhist,*hist;
@@ -1026,7 +1031,7 @@ int absupdate;
 	    strcpy(last.in,in);
 	    strcpy(last.out,out);
         }
-	fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now, inrate, outrate, inrate, outrate);
+       fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now, inrate, outrate, inrate, outrate);
 	nout = 1;
 	hist->time = now;
 	hist->in = inrate; hist->out = outrate;
@@ -1044,8 +1049,8 @@ int absupdate;
 	/* gobble up every shred of data we can get ... */
 	if (plannow < now) {
 	  NEXT((unsigned long) (now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		  (unsigned long)inr,(unsigned long)outr,(unsigned long)inmax,(unsigned long)outmax);
+         fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+                 (long long)inr,(long long)outr,(long long)inmax,(long long)outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
 	  hist->inmax = inmax; hist->outmax = outmax;
@@ -1056,8 +1061,8 @@ int absupdate;
 
 	for (x=1; x<DAY_COUNT; x++) {
 	    NEXT(DAY_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		(unsigned long)inr,(unsigned long)outr,(unsigned long)inmax,(unsigned long)outmax);
+           fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+               (long long)inr,(long long)outr,(long long)inmax,(long long)outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
 	    hist->inmax = inmax; hist->outmax = outmax;
@@ -1071,8 +1076,8 @@ int absupdate;
 
 	if (plannow < now) {
 	  NEXT((unsigned long) (now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
+         fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+                 (long long)inr,(long long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
 	  hist->inmax = inmax; hist->outmax = outmax;
@@ -1083,8 +1088,8 @@ int absupdate;
 	
 	for (x=0; x<WEEK_COUNT; x++) {
 	    NEXT(WEEK_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
+           fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+               (long long)inr,(long long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
 	    hist->inmax = inmax; hist->outmax = outmax;
@@ -1098,8 +1103,8 @@ int absupdate;
 
 	if (plannow < now) {
 	  NEXT((unsigned long) (now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
+         fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+                 (long long)inr,(long long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
 	  hist->inmax = inmax; hist->outmax = outmax;
@@ -1110,8 +1115,8 @@ int absupdate;
 
 	for (x=0; x<MONTH_COUNT; x++) {
 	    NEXT(MONTH_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
+           fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+               (long long)inr,(long long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
 	    hist->inmax = inmax; hist->outmax = outmax;
@@ -1125,8 +1130,8 @@ int absupdate;
 
 	if (plannow < now) {
 	  NEXT((unsigned long) (now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
+         fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+                 (long long)inr,(long long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
 	  hist->inmax = inmax; hist->outmax = outmax;
@@ -1137,8 +1142,8 @@ int absupdate;
 
 	for (x=0; x<YEAR_COUNT; x++) {
 	    NEXT(YEAR_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
-		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
+           fprintf(fo,"%lu " LLD " " LLD " " LLD " " LLD "\n",(unsigned long)now,
+               (long long)inr,(long long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
 	    hist->inmax = inmax; hist->outmax = outmax;
@@ -1305,8 +1310,8 @@ char **argv;
                break;
 	    case 'i':	/* Create PPM Image record */
 	      image(argv[argi+1], /* Image*/
-		    atol(argv[argi+2]),  /* Max Value In */
-                    atol(argv[argi+3]),  /* Max Value Out */
+                    strtoll(argv[argi+2],NULL,10),  /* Max Value In */
+                    strtoll(argv[argi+3],NULL,10),  /* Max Value Out */
  		    atol(argv[argi+4]),  /* xsize maxx */
 		    atol(argv[argi+5]),  /* ysize maxy */
 		    atof(argv[argi+6]),  /* xscale */
@@ -1321,8 +1326,8 @@ char **argv;
 	      break;
 	    case 'p':	/* Create PPM Image record with Peak values*/
 		image(argv[argi+1],
-		      atol(argv[argi+2]),  /* Max Value In */
-              atol(argv[argi+3]),  /* Max Value Out */
+                      strtoll(argv[argi+2],NULL,10),  /* Max Value In */
+                      strtoll(argv[argi+3],NULL,10),  /* Max Value Out */
 		      atol(argv[argi+4]),  /* xsize maxx */
 		      atol(argv[argi+5]),  /* ysize maxy */
 		      atof(argv[argi+6]),  /* xscale */
@@ -1344,35 +1349,35 @@ char **argv;
                 if (argv[argi][1] == 'p') {
 		   options |= OPTION_DORELPERCENT;
                 }
-		update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),0);
+               update(argv[argi+1],argv[argi+2],strtoll(argv[argi+3],NULL,10),0);
 		used = 4;
 		break;
 	    case 'a':	/* Absolute Update file */
                 if (argv[argi][1] == 'p') {
 		   options |= OPTION_DORELPERCENT;
                 }
-	   	update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),1);
+               update(argv[argi+1],argv[argi+2],strtoll(argv[argi+3],NULL,10),1);
 		used = 4;
 		break;
 	    case 'g':	/* Gauge Update file */
                 if (argv[argi][1] == 'p') {
 		   options |= OPTION_DORELPERCENT;
                 }
-		update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),2);
+               update(argv[argi+1],argv[argi+2],strtoll(argv[argi+3],NULL,10),2);
 		used = 4;
 		break;
             case 'h':
                 if (argv[argi][1] == 'p') {
 		   options |= OPTION_DORELPERCENT;
                 }
-                update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),3);
+                update(argv[argi+1],argv[argi+2],strtoll(argv[argi+3],NULL,10),3);
                 used = 4;
                 break;
             case 'm':
                 if (argv[argi][1] == 'p') {
 		   options |= OPTION_DORELPERCENT;
                 }
-                update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),4);
+                update(argv[argi+1],argv[argi+2],strtoll(argv[argi+3],NULL,10),4);
                 used = 4;
                 break;
 	    case 'W':	/* Week format */
